/*
=====================================
Program Name: AS1107				|
Software by: Thomas Kunzfeld		|
Begin of project: 22.2.2010			|
Tested on: ATMEGA32 @ 8Mhz			|
Compiler: AVR-GCC					|
Description: This program is able to|
			 communicate with 		|
			 several AS1107-ICs		|
			 Only 45 Deg. steps are |
			 possible				|
=====================================
*/

#define F_CPU 8000000L
//**********INCLUDES**************************************
#include <avr/io.h>
#include <stdbool.h>
#include <util/delay.h>
#include <avr/interrupt.h>
//**********DEFINITIONS***********************************

#define R_DISP_X 5		//Size of real Display
#define R_DISP_Y 7		//----||-----
#define V_DISP_X 8		//Max. displayable size
#define V_DISP_Y 8		//----||-----

bool display[V_DISP_X][V_DISP_Y];

#define UP		1
#define DOWN	2
#define LEFT	3
#define RIGHT	4

//*****PIN-DEFINITIONS************************************

#define DATADDR		DDRD //DDRD is data direction register for dataport
#define DATAPORT	PORTD//PORTD is dataport
#define DOUT 		PD6  //data-out, to DIN of AS1107
#define CLK			PD5  //clock, to CLK of AS1107
#define CS_IC1		PD4  //chip select, for 1st AS1107, has to 
						 //be low to select it; to CSN of AS1107

//*****MAKROS*********************************************

#define setbit(address, bit) 	(address |=  (1<<bit))
#define clearbit(address, bit)	(address &= ~(1<<bit))
#define checkbit(address, bit)	(address &   (1<<bit))

#define setclk()	setbit(PORTD,  CLK)  //set clock to high-signal
#define clearclk()	clearbit(PORTD, CLK)  //set clock to low-signal

#define selectIC1()		clearbit(PORTD, CS_IC1)  //CS has to be low to select IC
#define deselectIC1()	setbit(PORTD, CS_IC1)	 //CS is high->IC not selected

//*****STRUCTURES*****************************************

typedef struct _point
{
	unsigned char x, y;
	unsigned char speedX, speedY;
	unsigned char dirX, dirY;
	unsigned char counter;
	bool isStatic, start;
} point;

typedef struct _line
{
	unsigned char x, y;
	unsigned char length;
	point points[];
} line;

//**********FUNCTION-PROTOTYPES***************************

void AS1107_load(unsigned char address, unsigned char regData);
void avr_init();
void AS1107_init(bool test);
void display_test();
void display_init();
void display_draw();
uint8_t make_pos(int8_t num);

void render();
void move();

void point_draw(point p);
void point_move(point *p, bool checkCol);
void point_init(point *p, int8_t x, int8_t y, int8_t speedX, int8_t speedY, bool isStatic);
void point_col_check(point *p1, point *p2);

void line_init(line *L, uint8_t x, uint8_t y, uint8_t orientation, uint8_t length);
void line_draw(line *L);

//**********VARIABLES********************************************
point p1, p2;
line l1;








